;; Exp 2004/04/17 23:40:00 8.0 

(require 'proof)			; load generic parts

;; Adjust toolbar entries.  (Must be done
;; before proof-toolbar is loaded).

(if proof-running-on-XEmacs (setq nedpg-toolbar-entries
      (remassoc 'context nedpg-toolbar-entries)))


;; ======== User settings for Nedpg ========
;;
;; Defining variables using customize is pretty easy.
;; You should do it at least for your prover-specific user options.
;;
;; proof-site provides us with two customization groups
;; automatically:  (based on the name of the assistant)
;;
;; 'nedpg        -  User options for Nedpg Proof General
;; 'nedpg-config -  Configuration of Nedpg Proof General
;;			   (constants, but may be nice to tweak)
;;
;; The first group appears in the menu
;;   ProofGeneral -> Customize -> Nedpg 
;; The second group appears in the menu:
;;   ProofGeneral -> Internals -> Nedpg config
;;

(defcustom nedpg-prog-name "~/NED/ned"
  "*Name of program to run Nedpg."
  :type 'file
  :group 'nedpg)

(defcustom nedpg-sym-lock t
  "*Whether to use sym-lock or not."
  :type 'boolean
  :group 'nedpg)

(defcustom nedpg-web-page
  "http://www.ps.uni-sb.de/~huembert/bachelor.htm"
  "URL of web page for Nedpg."
  :type 'string
  :group 'nedpg-config)

(defcustom nedpg-doc-dir 
  ""
  "The name of the root documentation directory for Nedpg."
  :type 'string
  :group 'nedpg-config)

(defcustom nedpg-lib-dir 
  ""
  "The name of the root directory for Nedpg libraries."
  :type 'string
  :group 'nedpg-config)

(require 'nedpg-fun)
(require 'nedpg-font)
(require 'nedpg-outline)

;; default for tags [da: moved here to fix compilation 15/02/03]


;;
;; ======== Configuration of generic modes ========
;;

(defun nedpg-config ()
  "Configure Proof General scripting for Nedpg."
  (setq
   proof-terminal-char             ?\;
   proof-script-comment-start      "(*"
   proof-script-comment-end        "*)"
   proof-goal-command-regexp       "^goal"
   proof-save-command-regexp       "^qed"
   proof-goal-with-hole-regexp     "qed_goal \"\\(\\(.*\\)\\)\""
   proof-save-with-hole-regexp     "qed \"\\(\\(.*\\)\\)\""
   proof-non-undoables-regexp      "undo\\|back"
   proof-goal-command              "goal \"%s\";"
   proof-save-command              "qed \"%s\";"
   proof-kill-goal-command         "undo 1;"
   proof-showproof-command         "print;"
   proof-undo-n-times-cmd          "undo %s;"
   proof-auto-multiple-files       t
   proof-assistant-home-page       "http://www.ps.uni-sb.de/~huembert/bachelor.html"
   font-lock-keywords              (append nedpg-font-lock-keywords proof-xsym-font-lock-keywords)
   )
  (nedpg-init-syntax-table)
  (setq pbp-goal-command "intro %s;")
  (setq pbp-hyp-command "elim %s;")
;; the following is only useful for xemacs
  (define-key nedpg-mode-map [(meta ?.)] 'nedpg-complete-tag)
)

(defun nedpg-shell-config ()
  "Configure Proof General shell for Nedpg."
  (setq
   ;proof-shell-cd-cmd              "cd \"%s\""
   proof-shell-prompt-pattern      ">> "
   proof-shell-annotated-prompt-regexp  "\\(- \\)\\|\\(%nedpg% \\)"
   proof-shell-interrupt-regexp    "Interrupt"
   ;proof-shell-start-goals-regexp  "^BEGIN PRINTING"
   ;proof-shell-end-goals-regexp    "^END OF PRINTING"
   proof-shell-quit-cmd            "exit;"
   proof-shell-restart-cmd         "reset;"
   proof-shell-proof-completed-regexp   "PROOF DONE."
   ;; pg-subterm-first-special-char ?\371
   ;; proof-shell-wakeup-char ?\371
   ;; pg-subterm-start-char ?\371
   ;; pg-subterm-sep-char ?\372
   ;; pg-topterm-char ?\373
   ;; pg-subterm-end-char ?\374
   ;; proof-shell-eager-annotation-start "\376"
   ;; proof-shell-eager-annotation-start-length 1
   ;; proof-shell-eager-annotation-end "\377"
;    proof-shell-annotated-prompt-regexp "Lego> \371"
;    proof-shell-result-start "\372 Pbp result \373"
;    proof-shell-result-end "\372 End Pbp result \373"
;    proof-shell-start-goals-regexp "\372 Start of Goals \373"
;    proof-shell-end-goals-regexp "\372 End of Goals \373"
;    proof-shell-pre-sync-init-cmd "Configure AnnotateOn;"
))


;;
;; ======== Defining the derived modes ========
;;

;; The name of the script mode is always <proofsym>-script,
;; but the others can be whatever you like.
;;
;; The derived modes set the variables, then call the
;; <mode>-config-done function to complete configuration.

(define-derived-mode nedpg-mode proof-mode
    "Nedpg script" nil
    (nedpg-config)
    (nedpg-sym-lock-start)
    (proof-config-done)
    (nedpg-setup-outline)
    (define-key nedpg-mode-map [(control j)] 
      'nedpg-assert-next-command-interactive)
    ;; with the previous binding,
    ;; it is nice to do : xmodmap -e "keysym KP_Enter = Linefeed"

    (define-key nedpg-mode-map [(control c) (meta d)] 
      'nedpg-delete-symbol-on-cursor)  
)

(define-derived-mode nedpg-shell-mode proof-shell-mode
   "Nedpg shell" nil
   (nedpg-shell-config)
   (proof-shell-config-done))

(define-derived-mode nedpg-response-mode proof-response-mode
  "Nedpg response" nil
  (setq 
   font-lock-keywords (append nedpg-font-lock-keywords proof-xsym-font-lock-keywords)
   proof-output-fontify-enable     t)
  (nedpg-sym-lock-start)
  (if (and (featurep 'nedpg-sym-lock) nedpg-sym-lock)
      (add-hook 'proof-shell-handle-delayed-output-hook 
		'nedpg-sym-lock-font-lock-hook)
    )
  (proof-response-config-done))

(define-derived-mode nedpg-goals-mode proof-goals-mode
  "Nedpg goals" nil
  (setq 
   font-lock-keywords  (append nedpg-font-lock-keywords proof-xsym-font-lock-keywords)
   proof-output-fontify-enable     t)
  (nedpg-sym-lock-start)
  (if (and (featurep 'nedpg-sym-lock) nedpg-sym-lock)
      (add-hook 'pg-before-fontify-output-hook 
		'nedpg-sym-lock-font-lock-hook))
  (proof-goals-config-done))

;; The response buffer and goals buffer modes definedpg above are
;; trivial.  In fact, we don't need to define them at all -- they
;; would simply default to "proof-response-mode" and "pg-goals-mode".

;; A more sophisticated instantiation might set font-lock-keywords to
;; add highlighting, or some of the proof by pointing markup
;; configuration for the goals buffer.

;; The final piece of magic here is a hook which configures settings
;; to get the proof shell running.  Proof General needs to know the
;; name of the program to run, and the modes for the shell, response,
;; and goals buffers.

(add-hook 'proof-pre-shell-start-hook 'nedpg-pre-shell-start)

(defun nedpg-pre-shell-start ()
  (setq proof-prog-name		nedpg-prog-name)
  (setq proof-mode-for-shell    'nedpg-shell-mode)
  (setq proof-mode-for-response 'nedpg-response-mode)
  (setq proof-mode-for-goals	'nedpg-goals-mode))

; completions
; dans completions.el
;(setq completion-min-length 6)
;(setq completion-prefix-min-length 3) les mots de moins de 6 caractres
; ne sont pas pris en compte.  Les prefixes de moins de 3 caractres ne
; sont pas non plus pris en compte.

; (set-variable 'nedpg-completion-table 
(defpgdefault completion-table
  (append nedpg-top-keywords nedpg-proof-keywords)
)

(eval-after-load "x-symbol-nedpg"
 ;; Add x-symbol tokens to nedpg-completion-table and rebuild
 ;; internal completion table if completion is already active
'(progn
   (defpgdefault completion-table
     (append (proof-ass completion-table)
	     (mapcar (lambda (xsym) (nth 2 xsym))
		     x-symbol-nedpg-table)))
   (setq proof-xsym-font-lock-keywords
	 x-symbol-nedpg-font-lock-keywords)
   (if (featurep 'completion)
       (proof-add-completions))))

(provide 'nedpg)


