(* ETT.sml implementiert grundlegende Routinen zur 
 * Modifizierung einfach getypter Lambda terme 
 *   ! Arbeitet mit DeBrujin Darstellung !
 *)
local open Data in


(* Erhoeht die freien Variablen um y positionen *)
(* Wenn y = 1 ==> Standart up  *)
fun up l (C c) y             = C c
  | up l (V x) y             = V(if x>=l then x+y else x)
  | up l (g as (GAMMA _) ) y = g   
  | up l (A(m,m')) y         = A(up l m y, up l m' y)
  | up l (L((t,i),m)) y      = L((t,i), up (l+1) m y)
  | up l (EX((t,i),m)) y     = EX((t,i), up (l+1) m y)
  | up l (ALL((t,i),m)) y    = ALL((t,i), up (l+1) m y)
  | up l (AND(m,m')) y       = AND(up l m y, up l m' y)
  | up l (OR(m,m')) y        = OR(up l m y, up l m' y)
  | up l (IMP(m,m')) y       = IMP(up l m y, up l m' y)
  | up l (EQ(m,m')) y        = EQ (up l m y , up l m' y)
  | up l (NOT(m)) y          = NOT(up l m y)

(* Erniedrigt die freien Variablen um y positionen *)
(* Wenn y = 1 ==> Standart down  *)
fun down l (C c) y           = C c
  | down l (V x) y           = V(if x>=l then x-y else x)
  | down l (g as (GAMMA _)) y= g   
  | down l (A(m,m')) y       = A(down l m y , down l m' y)
  | down l (L((t,i),m)) y    = L((t,i), down (l+1) m y)
  | down l (EX((t,i),m)) y   = EX((t,i), down (l+1) m y)
  | down l (ALL((t,i),m)) y  = ALL((t,i), down (l+1) m y)
  | down l (AND(m,m')) y     = AND(down l m y , down l m' y)
  | down l (OR(m,m')) y      = OR(down l m y, down l m' y)
  | down l (IMP(m,m')) y     = IMP(down l m y, down l m' y)
  | down l (EQ(m,m')) y      = EQ(down l m y, down l m' y)
  | down l (NOT(m)) y        = NOT(down l m y)

fun sub (C c) y n            = C c
  | sub (V x) y n            = if x=y then n else V x
  | sub (g as (GAMMA _)) y n = g (* Substitution findet in Gamma.sml statt *)
  | sub (A(m,m')) y n        = A(sub m y n, sub m' y n)
  | sub (L((t,i),m)) y n     = L((t,i), sub m (y+1) (up 0 n 1))
  | sub (EX((t,i),m)) y n    = EX((t,i), sub m (y+1) (up 0 n 1))
  | sub (ALL((t,i),m)) y n   = ALL((t,i), sub m (y+1) (up 0 n 1))
  | sub (AND(m,m')) y n      = AND( sub m y n , sub m' y n )
  | sub (OR(m,m')) y n       = OR( sub m y n , sub m' y n )
  | sub (IMP(m,m')) y n      = IMP( sub m y n , sub m' y n )
  | sub (EQ(m,m')) y n       =  EQ( sub m y n, sub m' y n )
  | sub (NOT(m)) y n         = NOT( sub m y n )


fun free x (C c)             = false
  | free x (V y)             = x=y
  | free x (g as (GAMMA _) ) = false (* !!!! *)
  | free x (A(m,m'))         = free x m orelse free x m'
  | free x (L((t,i),m))      = free (x+1) m
  | free x (EX((t,i),m))     = free (x+1) m
  | free x (ALL((t,i),m))    = free (x+1) m
  | free x (AND(m,m'))       = free x m orelse free x m'
  | free x (OR(m,m'))        = free x m orelse free x m'
  | free x (IMP(m,m'))       = free x m orelse free x m'
  | free x (EQ(m,m'))        = free x m orelse free x m'
  | free x (NOT(m))          = free x m


(* Brauche neuen Gleichheitstest, wegen unterschiedlicher Variablennamen *)
fun equal ( (EX((t1,i1),f1)), (EX((t2,i2),f2)) )  = (t1=t2) andalso equal(f1,f2)
  | equal ( (ALL((t1,i1),f1)), (ALL((t2,i2),f2))) = (t1=t2) andalso equal(f1,f2)
  | equal ( (L ((t1,i1),f1)), (L ((t2,i2),f2)) )  = (t1=t2) andalso equal(f1,f2)
  | equal ( A(x1,y1) , A(x2,y2) )                 = equal(x1,x2) andalso equal(y1,y2)
  | equal ( AND(x1,y1) , AND(x2,y2) )             = equal(x1,x2) andalso equal(y1,y2)
  | equal ( OR(x1,y1) , OR(x2,y2) )               = equal(x1,x2) andalso equal(y1,y2)
  | equal ( IMP(x1,y1) , IMP(x2,y2) )             = equal(x1,x2) andalso equal(y1,y2)
  | equal ( EQ(x1,y1) , EQ(x2,y2) )               = equal(x1,x2) andalso equal(y1,y2)
  | equal ( NOT(m1) , NOT(m2) )                   = equal(m1,m2)
  | equal ( a, b)                                 = (a=b) 


(* Bekommt zusaetzliches Argument freelistlength zum richtigen umrechnen *)
(* Urspruenglich ist Mapping von [freexs]@[bndxs] *)
(* Spaeter soll aber Mapping nur noch auf bndxs beziehen !*) 
(*
fun com (C' c)          flength = C c
  | com (V' x)          flength = V x
  | com (A'(m,m'))      flength = A(com m flength, com m' flength)
  | com (L'((x,t),m))   flength = L((t,x-flength), sub (up 0 (com m flength) 1) (x+1) (V 0))
  | com (EX'((x,t),m))  flength = EX((t,x-flength), sub (up 0 (com m flength) 1) (x+1) (V 0))
  | com (ALL'((x,t),m)) flength = ALL((t,x-flength), sub (up 0 (com m flength) 1) (x+1) (V 0))
  | com (AND'(m,m'))    flength = AND(com m flength, com m' flength)
  | com (OR'(m,m'))     flength = OR(com m flength, com m' flength)
  | com (IMP'(m,m'))    flength = IMP(com m flength, com m' flength)
  | com (EQ'(m,m'))     flength = EQ(com m flength, com m' flength)
  | com (NOT'(m))       flength = NOT(com m flength)
*)

(* Ersetzt subterm p durch subterm n *)
fun termsub (t as (C c)) p n = if equal(p,t) then n else t
  | termsub (t as (V x)) p n = if equal(p,t) then n else t
  | termsub (t as ((GAMMA _))) p n = if equal(p,t) then n else t
  | termsub (t as (A(m,m'))) p n = if equal(p,t) then n else (A (termsub m p n, termsub m' p n) )
  | termsub (t as (L((typ,i),m))) p n = if equal(p,t) then n else (L((typ,i), termsub m (up 0 p 1) (up 0 n 1)))
  | termsub (t as (EX((typ,i),m))) p n = if equal(p,t) then n else (EX((typ,i), termsub m (up 0 p 1) (up 0 n 1)))
  | termsub (t as (ALL((typ,i),m))) p n = if equal(p,t) then n else (ALL((typ,i), termsub m (up 0 p 1) (up 0 n 1)))
  | termsub (t as (AND(m,m'))) p n = if equal(p,t) then n else (AND( termsub m p n , termsub m' p n ))
  | termsub (t as (OR(m,m'))) p n = if equal(p,t) then n else (OR( termsub m p n , termsub m' p n ))
  | termsub (t as (IMP(m,m'))) p n = if equal(p,t) then n else (IMP( termsub m p n , termsub m' p n ))
  | termsub (t as (EQ(m,m'))) p n = if equal(p,t) then n else (EQ( termsub m p n , termsub m' p n ))
  | termsub (t as (NOT(m))) p n = if equal(p,t) then n else (NOT( termsub m p n ))



(* Gibt eine Liste freier Variablen inkl. Gamma Variablen zurueck *)
fun getfreevars' (C c) n xs               = xs 
                                        (* muss V(y-n) ausgeben, da evtl. erhoeht *)
  | getfreevars' (V y) n xs               = if ( y >= n ) then ((V (y-n))::xs) else xs
  | getfreevars' (g as (GAMMA _) ) n xs   = (g::xs)
  | getfreevars' (A(m,m')) n xs           = (getfreevars' m n xs) @ (getfreevars' m' n xs)
  | getfreevars' (L((t,i),m)) n xs        = getfreevars' m (n+1) xs
  | getfreevars' (EX((t,i),m)) n xs       = getfreevars' m (n+1) xs
  | getfreevars' (ALL((t,i),m)) n xs      = getfreevars' m (n+1) xs
  | getfreevars' (AND(m,m')) n xs         = (getfreevars' m n xs) @ (getfreevars' m' n xs)
  | getfreevars' (OR(m,m')) n xs          = (getfreevars' m n xs) @ (getfreevars' m' n xs)
  | getfreevars' (IMP(m,m')) n xs         = (getfreevars' m n xs) @ (getfreevars' m' n xs)
  | getfreevars' (EQ(m,m')) n xs          = (getfreevars' m n xs) @ (getfreevars' m' n xs)
  | getfreevars' (NOT(m)) n xs            = (getfreevars' m n xs)
fun getfreevars term = getfreevars' term 0 nil


(* gibt eine Liste gebundener Variablen zurueck, die wegen Einsetzung,
   umbenannt werden muessen 
 *)

fun decide i m (bmap,fmap,cmap) l =
    let
	val bndname  = bmap i
	val freelist = getfreevars' m (l+1) nil
	val f = fn (p,result) => 
	    let
		val i = ( case p of
		         (V n) => n
		       | _ => (~1)  ) (* produziert Fehler *) 
		val fname = (fmap i) handle Error _ => ""
		val cname = (cmap i) handle Error _ => ""
	    in
		result orelse (bndname = fname) orelse (bndname = cname) 	
	    end
	val critical = foldl f false freelist
    in
	if critical then [i] else nil
    end

fun getbndvars' l (C c)            _    = nil
  | getbndvars' l (V x)            _    = nil
  | getbndvars' l (g as (GAMMA _)) _    = nil
  | getbndvars' l (A(m,m'))        maps = (getbndvars' l m maps) @ (getbndvars' l m' maps)
  | getbndvars' l (L((t,i),m))     maps = (decide i m maps l) @ (getbndvars' (l+1) m maps)
  | getbndvars' l (EX((t,i),m))    maps = (decide i m maps l)  @ (getbndvars' (l+1) m maps)
  | getbndvars' l (ALL((t,i),m))   maps = (decide i m maps l)  @ (getbndvars' (l+1) m maps)
  | getbndvars' l (AND(m,m'))      maps = (getbndvars' l m maps) @ (getbndvars' l m' maps)
  | getbndvars' l (OR(m,m'))       maps = (getbndvars' l m maps) @ (getbndvars' l m' maps) 
  | getbndvars' l (IMP(m,m'))      maps = (getbndvars' l m maps) @ (getbndvars' l m' maps)
  | getbndvars' l (EQ(m,m'))       maps = (getbndvars' l m maps) @ (getbndvars' l m' maps)
  | getbndvars' l (NOT(m))         maps = (getbndvars' l m maps) 

fun getCriticalBnds pter maps = getbndvars' 0 pter maps




end (* of local open *)

