(* Insert.sml implements the Equality Rule : Insertion 
 *    insert [l/r] name formula 
 * if M=N andalso type(name) = type(formula) 
 *   then M[name:=formula] = N[name:=formula] else Error
 *)


local open Data Goal in


fun main (goal, (d:xdirection, (name:string, f:sourceexpr)) ) = 
    let
	val pter = Goal.get_first_goal(goal)
	val (premise, conclusion) = 
	    (  case pter of 
		   IMP(l,r) => (l,r) 
		 | _ => raise Error ("Internal Error 1 Insert.sml\n") )

	(* Hole die betreffende Gleichung *)	    
	val (M,N) = ( case d of 
		      LEFT => ( case premise of 
			       EQ(a,b) => (a,b)
			     | AND(EQ(a,b),rest) => (a,b) 
			     | _ => raise Error ("There must be an equality, in which you can substitute "
						  ^ "Variable " ^ name ^ "\n")) 
		    | RIGHT => ( case conclusion of
				EQ(a,b) => (a,b)
			      | OR(EQ(a,b),rest) => (a,b) 
			      | _ => raise Error ("There must be an equality, in which you can substitute "
						  ^ "Variable " ^ name ^ "\n")) )

	 (* Hole Variablen Nr von name und Typ *)
	val varnr = ( Goal.freemaps2i(goal) ) name 
	val vartype = (Goal.typi(goal)) varnr

	val _ = if (ETT.free varnr M) andalso (ETT.free varnr N)
		    then () 
		else raise Error ("Equality does not contain free variable ["
				  ^ name ^ "] on both sides \n")

	(* Generiere Formel pter *)
	val insertinit = Goal.init_goal_env(goal)
	val insertgoal = Goal.first_goal(insertinit, f)
	val formel = ( case (Goal.get_first_goal(insertgoal)) of
		      IMP(C "TRUE",r) => r
		    | _ => raise Error ("Internal Error 4 Insert.sml") )

	(* Muss goals wieder mergen *)
	val mergegoal = Goal.merge_envs(goal, insertgoal)

	
	(* Checke jetzt die Types von name und formel *)
	val _ = if (Instance.checktype(formel, vartype) mergegoal) then () 
		else raise Error ("Sorry, types of name and insertion are different \n")

	(* Nehme nun Einfuegung vor *)
	val (M',N') = (ETT.sub M varnr formel, ETT.sub N varnr formel)

	val newpter = IMP ( case d of 
			   LEFT => ( (case premise of 
				      EQ _ => EQ(M',N')
				    | AND(EQ _ ,rest) => AND(EQ(M',N'),rest) 
				    | _ => raise Error "Internal Error 5 Insert.sml" ), conclusion)

			 | RIGHT =>(premise, ( case conclusion of
					      EQ _  => EQ(M',N')
					    | OR(EQ _ ,rest) => OR(EQ(M',N'),rest) 
					    | _ => raise Error ("Internal Error 6 Insert.sml"))) )
			       
	val newgoal' = Goal.update_first_goal(mergegoal, newpter)
	val newgoal =  Goal.kaper_check(newgoal')
    in
	(Goal.print_goal(newgoal); newgoal)
    end



end (* of local open *)

