(* Intro.sml implementiert alle Introduction-Regeln des Natural Deduction
 * calculus:
 *
 *  @  A => \/x . B[x]   -->  A => B[x]                 with x as a new parameter
 *  @  A => E x . B[x]   -->  A => B[? i]               (? i) is the i-th gamma variable
 *  @  A => B & C        -->  (1) A => B   (2) A => C   splits the goal in two subgoals
 *  @  A => ~B           -->  B & A => FALSE            let us assume B and show false (contradiction)
 *  @  A => B + C        -->  A => B  or  A => C        decide what to show ( <intro l> for B and <intro r> for C )
 *  @  A => ( B => C )   -->  A & B => C                extend the assumptions
 *  @  A => ( B = C )    -->  Equality substitutions vgl. "Equality.sml"
 *)

   
local open Data Goal in

fun main(g,modus) = let 
		val formel = Goal.get_first_goal(g)
		val new_goal = process formel g modus
	      in
		  (Goal.print_goal(new_goal);new_goal)
	      end

and process (IMP(left, ALL( (t,i), m))) goal _ =                          (* ALL RECHTS *)
     let 
	 (* holt den String namen der geb. Var i *)
	 val newname    = Goal.freeVarIntro(goal,i) 
	 (* hier werden ebenfalls die geb. Var ersetzt *)
	 val tempgoal = Goal.update_ftypeenv(goal, newname , t) 
	 val freemaps2i  = Goal.freemaps2i(tempgoal)

	 val tempgoal' = Goal.add_delta(tempgoal, freemaps2i newname)
	 val newright = ETT.down 0 (ETT.sub m 0 (V ((freemaps2i newname) + 1))) 1

	 val tempgoal'' = Goal.update_first_goal(tempgoal',IMP(left, newright ))
	 val newgoal = Goal.kaper_check(tempgoal'')     
     in
	 newgoal
     end

  | process (IMP(left, OR(ALL( (t,i), m), B))) goal _ =                    (* ALL RECHTS *)
     let 
	 (* holt den String namen der geb. Var i *)
	 val newname    = Goal.freeVarIntro(goal,i) 
	 (* hier werden ebenfalls die geb. Var ersetzt *)
	 val tempgoal = Goal.update_ftypeenv(goal, newname , t) 
	 val freemaps2i  = Goal.freemaps2i(tempgoal)

	 val tempgoal' = Goal.add_delta(tempgoal, freemaps2i newname)
	 val newright = ETT.down 0 (ETT.sub m 0 (V ((freemaps2i newname) + 1))) 1
	 val tempgoal''= Goal.update_first_goal(tempgoal',IMP(left, OR( newright , B)))
	 val newgoal = Goal.kaper_check(tempgoal'')
     in
	 newgoal
     end



  | process (IMP(left, oldright as (EX( (t,i), m)) )) goal modus =                              (* EXISTIERT RECHTS *)
     let
	 val nr_next_gamma = Goal.next_gamma_nr(goal)
	 val instance = GAMMA (t, nr_next_gamma)
	 val newgoal' = Goal.add_gamma(goal) 
	 val newright'= ETT.down 0 (ETT.sub m 0 (instance)) 1
	 val newright =( case modus of
			STANDART => newright'
		      | MORE => OR( newright', oldright ))
			     
	 val newgoal = Goal.update_first_goal(newgoal', IMP(left, newright) )
     in
	 newgoal
     end
  | process (IMP(left, oldright as (OR ( EX( (t,i), m) , B) ) )) goal modus =            (* EXISTIERT RECHTS *)
     let
	 val nr_next_gamma = Goal.next_gamma_nr(goal)
	 val instance = GAMMA (t, nr_next_gamma)
	 val newgoal' = Goal.add_gamma(goal)
	 val newright'= ETT.down 0 (ETT.sub m 0 (instance)) 1
	 val newright =( case modus of
			STANDART => OR(newright', B)
		      | MORE => OR( newright', oldright ))

	 val newgoal = Goal.update_first_goal(newgoal', IMP(left, newright) )
     in
	 newgoal
     end

  |process (IMP(left,AND(a,b))) goal _ =                                 (* AND Intro *)
     let
	 val g1 = Goal.update_first_goal(goal,IMP(left,a))
	 val g2 = Goal.split_goal(g1,IMP(left,b))
     in g2 
     end
  |process (IMP(left,OR(AND(a,b), B))) goal _ = 
     let
	 val g1 = Goal.update_first_goal(goal,IMP(left,OR(a,B)))
	 val g2 = Goal.split_goal(g1,IMP(left,OR(b,B)))
     in g2 
     end


  |process (IMP(left,NOT(a))) goal _ = 
     let 
	 val contradiction = 
	     (  case left of
		    (C "TRUE") => IMP(a, C "FALSE")
		  | _         => IMP(AND(a,left), C "FALSE" )
					     )
	 val g1 = Goal.update_first_goal(goal,contradiction)
     in g1 
     end
  |process (IMP(left,OR(NOT(a), B))) goal _ = 
     let 
	 val contradiction = 
	     (  case left of
		    (C "TRUE") => IMP(a, B)
		  | _         => IMP(AND(a,left), B )
					     )
	 val g1 = Goal.update_first_goal(goal,contradiction)
     in g1 
     end
 
 
  |process (IMP(left,IMP(a,b))) goal _ = 
     let 
	 val assume = 
	     (  case left of
		    (C "TRUE") => IMP(a, b)
		  | _         => IMP(AND(a,left), b)
			)
	 val g1 = Goal.update_first_goal(goal, assume )
     in g1 
     end
  |process (IMP(left,OR(IMP(a,b), B))) goal _ = 
     let 
	 val assume = 
	     (  case left of
		    (C "TRUE") => IMP(a, OR(b,B))
		  | _         => IMP(AND(a,left), OR(b,B))
			)
	 val g1 = Goal.update_first_goal(goal, assume )
     in g1 
     end
 
  |process (IMP(left,OR(OR(a,b), B))) goal _ =       (* Glaettet den OR Tree *)
     Goal.update_first_goal(goal,IMP(left,OR(b,OR(a, B))))

 
 

  |process (IMP(left, t as (EQ(a,b)) )) goal _ =
     let
	 (* Rufe Prozedur Equality.main auf
	  * gib den ersetzten Subterm zurueck
	  *)
	 val (newright, newgoal) = Equality.main(t, goal)  
     in
	 Goal.update_first_goal(newgoal, IMP(left, newright))
     end
	 
 |process (IMP(left, OR(t as (EQ(a,b)),B) )) goal _ =
     let
	 (* Rufe Prozedur Equality.main auf
	  * gib den ersetzten Subterm zurueck
	  *)
	 val (newright, newgoal) = Equality.main(t, goal)  
     in
	 Goal.update_first_goal(newgoal, IMP(left, OR(newright,B) ))
     end



(* TRANSLATE DEFINITION CHECK 
 *
 * Handelt es sich bei der Conklusion um eine Definition des users,
 * wird versucht, den Ausdruck der Definition einzusetzen: 
 * A => B[D]  and B contains a Definition (D=C) --> A => B[D:=C] 
 *)
  |process (IMP(left, right )) goal modus =   
     let
	 val (newright, manipulated) = 
	     case right of
		 OR(right',B) =>   
		     if (Goal.check_def(goal, right')) then (OR(Goal.translate_def(goal,right'),B), true)
		     else (right,false)
	       | _ => 
			 if (Goal.check_def(goal, right)) then (Goal.translate_def(goal,right), true)
			 else (right,false)

	 val newgoal = if manipulated then Goal.update_first_goal(goal, IMP(left, newright)) 
                       else goal
     in 
	 (* Damit kein Zyklus entsteht *)
	 if manipulated then 
	     let
		 val newfirst = Goal.get_first_goal(newgoal)
	     in 
		 process newfirst newgoal modus
	     end
	 else newgoal   
     end

  |process _ goal _ = goal

end (* of local open *)





