local open Data in


(* split wird in List.partition benutzt, um die goalxs 
 * in die Liste der Definitionen und der Kommandos zu teilen 
 *)
val split = fn (g,c) =>  
    ( case c of
	  LetVarS _    => true
        | LetConstS _  => true
	| DefS _       => true
	| TypS _       => true
	| EmptyS       => true
	| GoalS _      => false
        | SaveS _      => false
	| CmdS _       => false
	| CmdEqS _     => false
	| ForS _       => false
        | ErrorS _     => false
        | FileEndS     => false 
	      ) 

(* Returns a list of ( Goal and commando)  lists  *)
fun splitgoals' nil                       temp result = (temp::result)
  | splitgoals' ((t as (g,GoalS(_)))::xs) nil  result = splitgoals' xs [t] (result)
  | splitgoals' ((t as (g,GoalS(_)))::xs) temp result = splitgoals' xs [t] (temp::result)
  | splitgoals' (x::xs)                   temp result = splitgoals' xs (temp@[x]) result
fun splitgoals xs = List.rev (splitgoals' xs nil nil)

(* Hardcoded Header, falls Datei Header.tex nicht gefunden wird,
   nehme Standart Header 
 *)
val headerxs =
["%This is one possible proof output header\n",
 "% you can modify it\n",
 "\\documentclass[a4paper,10pt]{article}\n",
 "\\newtheorem{theorem}{Theorem}[section]\n",
 "\\newtheorem{lemma}[theorem]{Lemma}\n",
 "\\newtheorem{proposition}[theorem]{Proposition}\n",
 "\\newtheorem{corollary}[theorem]{Corollary}\n",
 "\\newenvironment{proof}[1][Proof]{\\begin{trivlist}\n",
 "\\item[\\hskip \\labelsep {\\bfseries #1}]}{\\end{trivlist}}\n",
 "\\newenvironment{definition}[1][Definitionen]{\\begin{trivlist}\n",
 "\\item[\\hskip \\labelsep {\\bfseries #1}]}{\\end{trivlist}}\n",
 "\\newenvironment{example}[1][Example]{\\begin{trivlist}\n",
 "\\item[\\hskip \\labelsep {\\bfseries #1}]}{\\end{trivlist}}\n",
 "\\newenvironment{remark}[1][Remark]{\\begin{trivlist}\n",
 "\\item[\\hskip \\labelsep {\\bfseries #1}]}{\\end{trivlist}}\n",
 "\\newcommand{\\qed}{\\nobreak \\ifvmode \\relax \\else\n",
 "     \\ifdim\\lastskip<1.5em \\hskip-\\lastskip\n",
 "     \\hskip1.5em plus0em minus0.5em \\fi \\nobreak\n",
 "     \\vrule height0.75em width0.5em depth0.25em\\fi}\n",
 "\n",
 "\\title{Proof Assistant NED Result}\n",
 "\\usepackage{amsmath,amssymb}\n",
 "\\begin{document}\n",
 "\\maketitle\n",
 "% END OF HEADER\n"] 

(* Hardcoded End, falls Datei End.tex nicht gefunden wird,
   nehme Standart Header 
 *)
val endxs = 
[ "% End Notes\n",
  "\\end{document}\n" ]

(* Kopiert den Inhalt der Header.tex / End.tex files in Output File *)
fun copy instream outstream =
    case TextIO.inputLine instream of
	"" => ()
      | s  => (TextIO.output(outstream,s); 
	       copy instream outstream )

fun copy' sxs outstream = foldl ( fn (s,_) => TextIO.output(outstream, s) ) () sxs



(* Zustaending fuer den Header der NEDOutput.tex *)
fun HeaderOutput outstream title =
( let 
      val instream = TextIO.openIn("../LatexOut/Header.tex") 
  in 
      ( copy instream outstream ;     
       TextIO.output(outstream, ("\\section{"^title^"}\n"));
       TextIO.closeIn instream 
       )
  end
) handle IO.Io _ => ( copy' headerxs outstream ; TextIO.output(outstream, ("\\section{"^title^"}\n")) )
    

(* Zustaending fuer den Bottom der NEDOutput.tex *)
fun EndOutput outstream = 
( let 
      val instream = TextIO.openIn("../LatexOut/End.tex")
  in 
      ( copy instream outstream ; 
       TextIO.closeIn instream  )
  end
) handle IO.Io _ => copy' endxs outstream


fun skipgoals g ((goal,CmdS(REMOVECMD _ ))::cx) = skipgoals goal cx
  | skipgoals g ((goal,CmdS(SHIFTCMD _ ))::cx)  = skipgoals goal cx
  | skipgoals g ((goal,SaveS(_))::cx) = skipgoals goal cx
  | skipgoals g cx = (g,cx)


fun DefinitionOutput _ nil = ()
 |  DefinitionOutput output defxs = 
    let 
	val _ = output ("\\begin{definition}:\\\\\n\\begin{itemize}")
	val _ = DefOut output (defxs)
	val _ = output ("\\end{itemize} \n\\end{definition}")
    in 
	()
    end
and DefOut output nil = ()
  | DefOut output (d::dx) = 
    let
	val (goal,cmd) = d
	val itemout = fn s => output( "\\item "^s )
    in
	(
	 ( case cmd of
	    LetVarS (s,t) => itemout ("Sei Konstante "^Visual.filter(s)^" : "^(Visual.ty2latex t) )
          | LetConstS (s,t) => itemout ("Sei Variable "^Visual.filter(s)^" : "^(Visual.ty2latex t) )
	  | DefS (s,_,k) => let
				val term = Goal.get_definition(goal,s)
				val prefix = ( case k of
					       CLASSIC => "Def "
					     | LEMMA   => "Lemma "
					     | Axiom   => "Axiom " )
			    in
				itemout (prefix ^ Visual.filter(s) ^ " = " ^(Goal.latex_goal(goal,term)))
			    end
	  | TypS (s)   => itemout ( "Neuer Typ : " ^ Visual.filter(s) )
	  | _ => ()
			  
	 );  (* end of case *)
	       DefOut output dx
	       )
    end



fun TheoremOutput output goalspez = 
    let
	val (goal,cmd) = goalspez 
	val _ = ( case cmd of 
	          (GoalS _ ) => ()
		 | _ => raise Error "No goal specification was found !\n"
  	        )
	     
	val term' = Goal.get_first_goal(goal)
	val term = case term' of
	             IMP(C "TRUE" , right ) => right
		   | t => t
        val _ = output("\\newpage")
	val _ = output("\n\\begin{theorem}:\\\\")
	val _ = output(Goal.latex_goal(goal,term))
	val _ = output("\\end{theorem}")
    in ()
    end



fun ProofOutput output nil = ()
 |  ProofOutput output cmdxs = 
    let 
	val _ = output ("\\begin{proof}:\\\\\\\\")
	val _ = ProofOut output (cmdxs)
	val _ = output ("\\hspace*{5mm} $\\top$ \\\\\n\\qed\n\\end{proof}")
    in 
	()
    end
and ProofOut output nil = ()
  | ProofOut output (c::cx) = 
    let
	val (goal,cmd) = c
	val _ =    
	 ( case cmd of
	       CmdS (t)   =>  output ("$\\dashv $ {\\tiny "^Visual.comm2string(t)^"}\\\\\\\\")
	     | CmdEqS (t) =>  output ("$\\dashv $ {\\tiny "^Visual.commeq2string(t)^"}\\\\\\\\")
	     | _ => ()
	  ) (*end of case*)
	val (goal', cx') = skipgoals goal cx 
    in
	( 
	 ( let
	      val actives = (Goal.get_nr_of_goals(goal'))
	   in  
	       if (actives > 0 ) then 
		   if ( Goal.get_first_goal(goal') <> (C "PROOF DONE") )
		       then ( ProofOutSubgoals output goal' (1, actives) ) else ()
		   else ()
	   end
	  );   (* end of case *)
	 
 	  ProofOut output cx'
       )
    end

and ProofOutSubgoals output goal (nr, actives) = 
    let 
	val term = Goal.get_first_goal(goal)  
	(* val _ = if (nr > 1) then (output (Int.toString(nr)^ " Subgoals remaining \\\\")) else () *)
	val _ = output (Goal.latex_goal(goal, term) ^ "\\\\")
    in
	if (* (nr < actives ) *)
	   ( nr < actives )
	    then
		(output (Int.toString(actives - nr) ^ " Subgoal" ^ 
			 (if (actives-nr) > 1 then "s" else "") ^ 
			      " remaining \\\\"))
		(* ProofOutSubgoals output (Goal.remove_first_goal(goal)) ( nr+1 , actives) *)
	else ()
    end

	 

fun GoalOutput output nil = ()
 |  GoalOutput output (x::xs) =
      ( TheoremOutput output (hd(x)) ;
        ProofOutput   output (tl(x)) ;
        GoalOutput output xs          )
      
(* Start here *)
fun main(goalxs,title) = 
let
    val outstream = TextIO.openOut("NEDOutput.tex")
    (* output is a function that prints out s and a carriage return *)
    val output = fn s => TextIO.output(outstream, s^"\n")  
    (* splittet die goalxs in 
       - Preprocessing Commandos ( like Definitions, typedeclarations , var def
       - Goal Spezifikation und Kommandos zur Bearbeitung
     *)

    (* Reversiere deswegen, weil das erste Element der Liste auch
     * das erste Kommando sein soll 
     *)
    val (prexs, cmdxs) = List.partition split ( List.rev (goalxs) )
    val (goalxs) = splitgoals (cmdxs)

in 
    ( HeaderOutput outstream title;
      DefinitionOutput output prexs;
      GoalOutput output goalxs;
      EndOutput outstream ;
      TextIO.closeOut outstream )
end


end (* of local open *)				    