(* 
 *   Pretty Printer for HOL Formula
 *)
local open Data PP in 

(* Ausdruecke werden mit moeglichst wenigen Klammern 
 * geprintet;
 *  fmap - stellt die Variablenumgebung der freien Variablen dar ( int -> string )
 *  bmap - Variablenumgebung der gebundenen Variablen ( int -> string ) 
 *  vmap - wird dynamisch angepasst V 0 -> neu gebundene Variable  
 *)


val initvmap = fn x:int => raise Error (" INIT VMAP ERROR !! ")
fun cons(f,s) = fn x:int => if x = 0 then s else f(x-1)


fun ppexpr (fmap,bmap) pps e0 = 
    let 
	val vmap = initvmap


	fun exp (L((t,i),e))   fmap bmap vmap l = (begin_block pps INCONSISTENT 0;
						   add_string pps "\\ ";
						   add_string pps (bmap i);
						   add_string pps ". ";
						   add_break pps (0, 1); 
						   exp e fmap bmap (cons(vmap,bmap i)) (l+1);
						   end_block pps)
	  | exp (EX((t,i),e))  fmap bmap vmap l = (begin_block pps CONSISTENT 0;
						   add_string pps "/\\ ";
						   add_string pps (bmap i);
						   add_string pps ". ";
						   (* add_break pps (0, 1); *)
						   exp e fmap bmap (cons(vmap,bmap i)) (l+1);
						   end_block pps)
	  | exp (ALL((t,i),e)) fmap bmap vmap l = (begin_block pps CONSISTENT 0;
						   add_string pps "\\/ ";
						   add_string pps (bmap i);
						   add_string pps ". ";
						   (* add_break pps (0, 1); *)
						   exp e fmap bmap (cons(vmap,bmap i)) (l+1);
						   end_block pps)
	  | exp t fmap bmap  vmap l = eqexp t fmap bmap vmap l
	    
	and eqexp (EQ(a,b)) fmap bmap vmap l =  (begin_block pps CONSISTENT 0;
						 impexp a fmap bmap vmap l; 
						 add_string pps " = ";
						 add_break pps (0, 1); 
						 eqexp b fmap bmap vmap l;
						 end_block pps)
	  | eqexp t         fmap bmap vmap l = impexp t fmap bmap vmap l 
	    
	and impexp (IMP(a,b))   fmap bmap  vmap l = (begin_block pps CONSISTENT 0;
						     orexp a fmap bmap vmap l; 
						     add_string pps " => ";
						     add_break pps (0, 1); 
						     impexp b fmap bmap vmap l;
						     end_block pps)
	  | impexp t            fmap bmap  vmap l = orexp t fmap bmap vmap l
	    
	and orexp (OR(a,b)) fmap bmap  vmap l = (begin_block pps CONSISTENT 0;
						     andexp a fmap bmap vmap l; 
						     add_string pps " + ";
						     add_break pps (0, 1); 
						     orexp b fmap bmap vmap l;
						     end_block pps)
	  | orexp t         fmap bmap  vmap l = andexp t fmap bmap vmap l
	    
	and andexp (AND(a,b)) fmap bmap  vmap l = (begin_block pps CONSISTENT 0;
						     negexp a fmap bmap vmap l; 
						     add_string pps " & ";
						     add_break pps (0, 1); 
						     andexp b fmap bmap vmap l;
						     end_block pps)
	  | andexp t          fmap bmap  vmap l = negexp t fmap bmap vmap l
	    
	and negexp (NOT(a)) fmap bmap  vmap l = (add_string pps "~ " ; negexp (a) fmap bmap vmap l)
	  | negexp t        fmap bmap  vmap l = appexp t fmap bmap vmap l
	    
	and appexp (A(a,b)) fmap bmap  vmap l =(begin_block pps CONSISTENT 0;
						     appexp a fmap bmap vmap l; 
						     add_string pps " ";
						     (* add_break pps (0, 1); *)
						     atexp b fmap bmap vmap l;
						     end_block pps)
 	  | appexp t        fmap bmap  vmap l = atexp t fmap bmap vmap l
	    
	and atexp (V(n)) fmap bmap  vmap l = add_string pps (if ( n >= l ) then fmap (n-l) else vmap n)
	  | atexp (GAMMA(_,i)) fmap bmap vmap l = add_string pps ("? "^Int.toString(i))
	  | atexp (C(s)) fmap bmap  vmap l = add_string pps s
	  | atexp t      fmap bmap  vmap l = (begin_block pps CONSISTENT 0;
					      add_string pps "( " ;
					      exp t fmap bmap vmap l ;
					      add_string pps  " )" ;
					      end_block pps )
	    
	    

    in
        begin_block pps INCONSISTENT 0; 
        exp e0 fmap bmap vmap 0;
        end_block pps
    end


(* For Meta installation val _ = installPP ppexpr; *)

fun getPPString term maps = pp_to_string 80 (ppexpr maps) term 


(* For Equality Deduction Mode *)
(* ohne Pretty Printing, aber mit Positions Labels *)

fun ppequal pter (fmap,bmap) = 
    let 
	val vmap = initvmap
	val label = ref 65 (* starte bei #"A" *)

	fun putLabel (label, size) = 
	    let
		val label2string = Char.toString(Char.chr(label))
	        fun blanks i = (implode (List.tabulate (i, (fn n => #" "))))
	    in ( label2string ^ (if size < 0 then "" else (blanks size)) )
	    end
	    
	fun exp (L((t,i),e))   (fmab, bmap, vmap) l = 
	    let val clabel = (!label) before (label := !label + 1)
		val (rets, retl) = ( exp e (fmap, bmap, (cons(vmap,bmap i))) (l+1))

		val fs  = "\\ " ^ (bmap i) ^ ". " ^ rets
		val ls  = ( putLabel (clabel, 3 + String.size(bmap i)) ) ^ retl
	    in (fs, ls)
	    end
	
	  | exp (EX((t,i),e))   (fmab, bmap, vmap) l = 
	    let val clabel = (!label) before (label := !label + 1)
		val (rets, retl) = ( exp e (fmap, bmap, (cons(vmap,bmap i))) (l+1))

		val fs  = "/\\ " ^ (bmap i) ^ ". " ^ rets
		val ls  = ( putLabel (clabel, 4 + String.size(bmap i)) ) ^ retl
	    in (fs, ls)
	    end

	  | exp (ALL((t,i),e))   (fmab, bmap, vmap) l = 
	    let val clabel = (!label) before (label := !label + 1)
		val (rets, retl) = ( exp e (fmap, bmap, (cons(vmap,bmap i))) (l+1))

		val fs  = "\\/ " ^ (bmap i) ^ ". " ^ rets
		val ls  = ( putLabel (clabel, 4 + String.size(bmap i)) ) ^ retl
	    in (fs, ls)
	    end

	  | exp t maps l = eqexp t maps l
	    
	and eqexp (EQ(a,b)) maps l =
	    let val clabel = (!label) before (label := !label + 1)
		val (rets1, retl1) = ( impexp a maps l ) 
		val s = " = " 
		val (rets2, retl2) = ( eqexp b maps l )

		val fs = rets1 ^ s ^ rets2
		val ls = retl1 ^ " " ^ putLabel(clabel, 1) ^ retl2
	    in (fs, ls)
	    end
						 
	  | eqexp t maps l = impexp t maps l 
	    
	and impexp (IMP(a,b)) maps l =
	    let val clabel = (!label) before (label := !label + 1)
		val (rets1, retl1) = ( orexp a maps l ) 
		val s = " => " 
		val (rets2, retl2) = ( impexp b maps l )

		val fs = rets1 ^ s ^ rets2
		val ls = retl1 ^ " " ^ putLabel(clabel, 2) ^ retl2
	    in (fs, ls)
	    end
	  | impexp t maps l = orexp t maps l
	    
	and orexp (OR(a,b)) maps l =
	    let val clabel = (!label) before (label := !label + 1)
		val (rets1, retl1) = ( andexp a maps l ) 
		val s = " + " 
		val (rets2, retl2) = ( orexp b maps l )

		val fs = rets1 ^ s ^ rets2
		val ls = retl1 ^ " " ^ putLabel(clabel, 1) ^ retl2
	    in (fs, ls)
	    end

	  | orexp t maps l = andexp t maps l
	    
	and andexp (AND(a,b)) maps l =
	    let val clabel = (!label) before (label := !label + 1)
		val (rets1, retl1) = ( negexp a maps l ) 
		val s = " & " 
		val (rets2, retl2) = ( andexp b maps l )

		val fs = rets1 ^ s ^ rets2
		val ls = retl1 ^ " " ^ putLabel(clabel, 1) ^ retl2
	    in (fs, ls)
	    end

	  | andexp t maps l = negexp t maps l
	    
	and negexp (NOT(a)) maps l =
	    let val clabel = (!label) before (label := !label + 1)
		val (rets, retl) = ( negexp a maps l ) 
		val s = "~ " 

		val fs = s ^ rets
		val ls = putLabel(clabel, 1) ^ retl
	    in (fs, ls)
	    end

	  | negexp t maps l = appexp t maps l
	    
	and appexp (A(a,b)) maps l = 
	    let val (rets1, retl1) = appexp a maps l
		val (rets2, retl2) = atexp b maps l 
		val fs = rets1 ^ " " ^ rets2
		val ls = retl1 ^ " " ^ retl2
	    in (fs, ls)
	    end

	  | appexp t maps l = atexp t maps l
	    
	and atexp (V(n)) (fmap,_ , vmap) l = 
	    let val clabel = (!label) before (label := !label + 1)
		val s = (if ( n >= l ) then fmap (n-l) else vmap n)
		val size = String.size(s)
	    in (s, putLabel(clabel, size - 1))  (* put clabel instead 32 to label variables also *)
	    end
	  | atexp (GAMMA(_,i)) _ _ = 
	    let val i2s = Int.toString(i) 
		val s  = ("? " ^ i2s)
	    in (s, putLabel(32, 1 + String.size(i2s))) 
	    end
	  | atexp (C(s)) _ _ = 
	    let val clabel = (!label) before (label := !label + 1)
		val size = String.size(s)
	    in (s, putLabel(clabel, size - 1)) (* put clabel instead 32 to label constants also *)
	    end
	  | atexp t maps l = 
	    let val (rets, retl) = exp t maps l 
		val fs = "( " ^ rets ^ " )"
		val ls = "  " ^ retl ^ "  "
	    in (fs, ls)
	    end
    in
	(label := 65; 
	 exp pter (fmap, bmap, vmap) 0)
    end


fun getEqualStrings term maps = ppequal term maps


end; (* of local open *) 
