(* This Remove.sml Modul implements the Removing Procedure of 
 * an Hypothesis/Conclusion :
 * <remove (l/r) i> - removes the i-th Hypothesis in the Conjunctiontree 
 *                                    /Conclusion in the Disjunctiontree
 *)

local open Data Goal in
exception norem


(* Removes the n.th component in an AND Tree *)
fun removeit_and' (AND(a,b)) n l = if n=l then b 
			       else( case b of
					AND(x,y) => AND(a,removeit_and' b n (l+1))
				      | t => if ( n=(l+1) ) then a else AND(a,b) )
					
  | removeit_and' _ _ _ = raise norem
fun removeit_and p n = removeit_and' p n 1

(* Removes the n.th component in an OR Tree *)
fun removeit_or' (OR(a,b)) n l = if n=l then b 
			       else ( case b of
					OR(x,y) => OR(a,removeit_or' b n (l+1))
				      | t => if ( n=(l+1) ) then a else OR(a,b) )
					
  | removeit_or' _ _ _ = raise norem
fun removeit_or p n = removeit_or' p n 1



(* **********  Main Processes *********** *)

fun process_left (IMP(left as AND(a,B), right)) nr goal = 
    let
	val newleft = (removeit_and left nr) handle norem => left
	val newgoal = Goal.update_first_goal(goal,IMP(newleft ,right) )
    in newgoal 
    end
  |process_left (IMP(a, right)) 1 goal = 
    let
	val newgoal = Goal.update_first_goal(goal, IMP(C "TRUE" ,right) )
    in newgoal 
    end
  |process_left t _ goal = goal

fun process_right (IMP(left, right as OR(a,B))) nr goal = 
    let
	val newright = (removeit_or right nr) handle norem => right
	val newgoal = Goal.update_first_goal(goal,IMP(left , newright) )
    in newgoal 
    end
  |process_right (IMP(left, a)) 1 goal = 
    let
	val newgoal = Goal.update_first_goal(goal,IMP(left , C "FALSE" ) )
    in newgoal 
    end
  |process_right t _ goal = goal



fun main(g,(d,i)) = 
    let 
	val formel = Goal.get_first_goal(g)
	val new_goal = case d of 
	    LEFT  => process_left formel i g
	  | RIGHT => process_right formel i g
    in
	(Goal.print_goal(new_goal);new_goal)
    end

end (* of local open *)

