(* Shifter Routinen der Praemissen :
 * <shift l u [nr] > - shiftet die Praemissen zyklisch um eine Position nach oben 
 *                   [ optional um Anzahl <nr> Positionen ] 
 * <shift l d [nr] > - analog nach unten
 *
 * <shift r u [nr] > - shiftet die Conclusionen zyklisch um eine Position nach oben 
 *                   [ optional um Anzahl <nr> Positionen ] 
 * <shift r d [nr] > - analog nach unten
 *
 *
 * <shift goals [nr] > - shiftet die Subgoals cyclisch um eine Position nach oben 
 *                    [ optional um Anzahl <nr> Positionen ] 
 *)

local open Data Goal in
exception never
(* Go to the right most leaf in AND Tree and put m there *)
fun putlast_and (AND(x,y)) m = AND(x,putlast_and y m)
  | putlast_and t m = AND(t,m)

(* Returns the right most leaf in Front of an AND Tree *)
fun getlast_and (AND(x,t as AND(y,z))) = let
					     val (a,b) = ( case (getlast_and t) of
							       AND(a,b) => (a,b)
							      | _ => raise never
							 )
					 in
					     AND(a,AND(x,b))
					 end
  | getlast_and (AND(x, leaf ))        = AND(leaf,x)
  | getlast_and t = t

(* Go to the right most leaf in OR Tree and put m there *)
fun putlast_or (OR(x,y)) m = OR(x,putlast_or y m)
  | putlast_or t m = OR(t,m)

(* Returns the right most leaf in Front of an OR Tree *)
fun getlast_or (OR(x,t as OR(y,z))) = let
                                       val (a,b) = ( case (getlast_or t) of
							       OR(a,b) => (a,b)
							      | _ => raise never
						   )
				     in
					 OR(a,OR(x,b))
				     end
  | getlast_or (OR(x, leaf ))        = OR(leaf,x)
  | getlast_or t = t

(* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)

(* Force left a Cyclic Left Shift        1 & 2 & 3   ->   2 & 3 & 1      *)
fun shift_left_up (IMP(AND(a,B),right)) goal =  
    let
	val newleft = putlast_and B a
    in
	Goal.update_first_goal(goal, IMP(newleft,right) )
    end
  | shift_left_up _ goal = goal

    
(* Force left a Cyclic Right Shift        1 & 2 & 3   ->   3 & 1 & 2      *)
fun shift_left_down (IMP(AND(a,B),right)) goal = 
    let
	val newleft = getlast_and (AND(a,B))
    in
	Goal.update_first_goal(goal, IMP(newleft,right) )
    end
  | shift_left_down _ goal = goal


(* ********************  Rechts Shifting ********************** *)

(* Force right a Cyclic Left Shift        1 & 2 & 3   ->   2 & 3 & 1      *)
fun shift_right_up (IMP(left,OR(a,B))) goal =  
    let
	val newright = putlast_or B a
    in
	Goal.update_first_goal(goal, IMP(left,newright) )
    end
  | shift_right_up _ goal = goal

    
(* Force right a Cyclic Right Shift        1 & 2 & 3   ->   3 & 1 & 2      *)
fun shift_right_down (IMP(left,OR(a,B))) goal = 
    let
	val newright = getlast_or (OR(a,B))
    in
	Goal.update_first_goal(goal, IMP(left,newright) )
    end
  | shift_right_down _ goal = goal


(* fuer shifings um mehrere Positionen *)
fun loopshift cmd goal 0 = goal
  | loopshift cmd goal i = 
    let val pter = Goal.get_first_goal(goal)
    in loopshift cmd (cmd pter goal) (i-1)
    end

fun goalloopshift goal 0 = goal
  | goalloopshift goal i = goalloopshift (Goal.cyclic_shift(goal)) (i-1)


fun main(goal, x, (y, i) ) = let
			val pter = Goal.get_first_goal(goal)
			val newgoal = case x of  (* X-Richtung, LEFT or RIGHT *)
			    LEFT => ( case y of   (* Y-Richtung also up and down *)
				     UP   => loopshift (shift_left_up)   goal i
				   | DOWN => loopshift (shift_left_down) goal i
					 )
			  |RIGHT =>  ( case y of   (* Y-Richtung also up and down *)
				     UP   => loopshift (shift_right_up)   goal i
				   | DOWN => loopshift (shift_right_down) goal i
					 )
		    in
			(Goal.print_goal(newgoal); newgoal)
		    end



fun goalmain(goal, n) = 
    let
	val newgoal = goalloopshift goal n
    in
	(Goal.print_goal(newgoal); newgoal)
    end

end (* of local open *)